package net.ihe.gazelle.proxy.netty.protocols.tlsRaw;

import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.ConnectionConfigSimple;
import net.ihe.gazelle.proxy.netty.Proxy;
import net.ihe.gazelle.proxy.netty.ProxyEventListener;
import org.junit.Ignore;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Timestamp;
import java.util.ArrayList;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

/**
 * Created by gthomazon on 19/09/16.
 */
public class HandleReceivedMessagesTest {

    private static Logger LOG = LoggerFactory.getLogger(HandleReceivedMessagesTest.class);

    private static void printSocketInfo(SSLSocket s) {
        System.out.println("Socket class: " + s.getClass());
        System.out.println("   Remote address = "
                + s.getInetAddress().toString());
        System.out.println("   Remote port = " + s.getPort());
        System.out.println("   Local socket address = "
                + s.getLocalSocketAddress().toString());
        System.out.println("   Local address = "
                + s.getLocalAddress().toString());
        System.out.println("   Local port = " + s.getLocalPort());
        System.out.println("   Need client authentication = "
                + s.getNeedClientAuth());
        SSLSession ss = s.getSession();
        System.out.println("   Cipher suite = " + ss.getCipherSuite());
        System.out.println("   Protocol = " + ss.getProtocol());
    }

    public static byte[] concatinateBytes(byte[] left, byte[] right) {
        byte[] newArray = new byte[left.length + right.length];
        System.arraycopy(left, 0, newArray, 0, left.length);
        System.arraycopy(right, 0, newArray, left.length, right.length);

        return newArray;
    }

    @Test
    public void testGivenACompleteMessageReturnsTheMessage() {
        byte[] completeMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x01, (byte) 0x01};

        HandleReceivedMessages hrm = new HandleReceivedMessages();
        hrm.push(completeMessage);
        ArrayList<byte[]> messages = hrm.splitMessages();
        assertEquals(1, messages.size());
        assertArrayEquals(completeMessage, messages.get(0));
    }

    @Test
    public void testGivenACompleteMessageReturnsTheMessageOnce() {
        byte[] completeMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x01, (byte) 0x01};

        HandleReceivedMessages hrm = new HandleReceivedMessages();
        hrm.push(completeMessage);
        ArrayList<byte[]> messages = hrm.splitMessages();
        assertEquals(1, messages.size());
        assertArrayEquals(completeMessage, messages.get(0));

        messages = hrm.splitMessages();
        assertEquals(0, messages.size());
    }

    @Test
    public void testGivenMultipleCompleteMessagesReturnsAnArrayOfMessages() {
        byte[] completeMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x01, (byte) 0x01};
        byte[] completeMessage2 = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x02};

        byte[] twoMessages = concatinateBytes(completeMessage, completeMessage2);
        HandleReceivedMessages hrm = new HandleReceivedMessages();
        hrm.push(twoMessages);
        ArrayList<byte[]> messages = hrm.splitMessages();
        assertEquals(2, messages.size());
        assertArrayEquals(completeMessage, messages.get(0));
        assertArrayEquals(completeMessage2, messages.get(1));

        messages = hrm.splitMessages();
        assertEquals(0, messages.size());
    }

    @Test
    public void testGivenAnIncompleteMessageItReturnsNothing() {
        byte[] incompleteMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x01};

        HandleReceivedMessages hrm = new HandleReceivedMessages();
        hrm.push(incompleteMessage);
        ArrayList<byte[]> messages = hrm.splitMessages();
        assertEquals(0, messages.size());
    }

    @Test
    public void testGivenAnIncompleteMessageAndItsEndReturnsMessages() {
        byte[] incompleteMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x01};
        byte[] endOfMessagePlusOneMessage = {(byte) 0x02, (byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x02};

        byte[] expectedFirstMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x02};
        byte[] expectedSecondMessage = {(byte) 0x16, (byte) 0x03, (byte) 0x03, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x02};


        HandleReceivedMessages hrm = new HandleReceivedMessages();
        hrm.push(incompleteMessage);
        hrm.push(endOfMessagePlusOneMessage);
        ArrayList<byte[]> messages = hrm.splitMessages();
        assertEquals(2, messages.size());

        assertArrayEquals(expectedFirstMessage, messages.get(0));
        assertArrayEquals(expectedSecondMessage, messages.get(1));

        messages = hrm.splitMessages();
        assertEquals(0, messages.size());
    }

    @Ignore
    public void tlsThroughNettyShouldSplitMessages() throws IOException, InterruptedException {
        ConnectionConfigSimple connectionConfig = new ConnectionConfigSimple(10001, "127.0.0.1", 1443, ChannelType.TLSRAW);
        ProxyEventListener proxyEventListener = new ProxyEventListener() {
            @Override
            public void onRequest(Timestamp timeStamp, Object request, String requesterIp, int requesterPort, int proxyProviderPort, String responderIp, int responderPort, int providerChannelId, int consumerChannelId) {
            }

            @Override
            public void onResponse(Timestamp timeStamp, Object response, String requesterIp, int requesterPort, int proxyProviderPort, String responderIp, int responderPort, int providerChannelId, int consumerChannelId) {
            }
        };
        Proxy<?, ?> proxy = new TlsRawProxy(proxyEventListener, connectionConfig);

        proxy.start();
        LOG.warn(proxy.getInfo());
        connectOpenSSl();
    }

    public void connectOpenSSl() throws IOException, InterruptedException {
        LOG.warn("connectOpenSSl");
        Process p;

        String command = "/home/gthomazon/workspace/gazelle-proxy/gazelle-proxy-netty/src/test/resources/start_client_via_proxy.sh";

        p = Runtime.getRuntime().exec(command);

        Thread.sleep(1000);

        p.destroy();
    }

    class StreamReader implements Runnable {
        private final InputStream in;
        private final OutputStream out;


        public StreamReader(InputStream in, OutputStream out) {
            this.in = in;
            this.out = out;
        }

        @Override
        public void run() {
            int c;
            try {
                while ((c = in.read()) != -1) {
                    out.write(c);
                }
                out.flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}